module Manager.UpdateManager exposing (update)
-- Browser
import Browser
import Browser.Navigation as Nav
-- Http
import Http 
-- Url
import Url
-- Own
import TypeHolder exposing (Model, Msg(..))
import Manager.ModalManager exposing (..)
import Util.ShapeManager exposing (..)
import Util.Util         exposing (..)

-- UPDATE
update : Msg -> Model -> (Model, Cmd Msg)
update msg model = 
   (case msg of 
      OpenModal    modalMsg       -> updateModal modalMsg model
      OnInputModal modalMsg value -> updateModal (case modalMsg of
                                                  Check         _           -> modalMsg
                                                  Input         inputType _ -> Input inputType value
                                                  InputPoint                -> modalMsg
                                                  AddShape                  -> modalMsg
                                                  DeleteShape   _           -> modalMsg
                                                  OpenShowInput             -> modalMsg
                                                  OpenShowTable             -> modalMsg
                                                  OpenShowShape _           -> modalMsg
                                                  OpenDrawShape _           -> modalMsg) model
      CloseModal                  -> { model | modal = NoModal }
      SvgShapeHover id            -> { model | svgShapeHover = id}
      UpdateMousePos mouse -> case model.modal of
          NoModal         -> model -- ERROR
          ShowTable       -> model -- ERROR       
          ShowShape _     -> model -- ERROR
          ShowInput _ _ _ -> model -- ERROR
          DrawShape iShape mOld -> { model | modal = DrawShape iShape mouse }
      InputDrawShape -> case model.modal of
          NoModal         -> model -- ERROR
          ShowTable       -> model -- ERROR       
          ShowShape _     -> model -- ERROR
          ShowInput _ _ _ -> model -- ERROR
          DrawShape iShape mPos -> case iShape of
              Empty             -> model -- ERROR
              Circle    x y r   -> 
                if x=="" then { model | modal = DrawShape (Circle (String.fromInt mPos.x) 
                                                                  (String.fromInt mPos.y) r) mPos}
                         else { model | saved = (Circle x y (abstand x y mPos))::model.saved
                                      , modal = NoModal
                              }
              Rectangle a b w h -> 
                if a=="" 
                  then { model | modal = DrawShape 
                                           (Rectangle (String.fromInt mPos.x) (String.fromInt mPos.y) w h)
                                            mPos
                       }
                  else let coords = manageRectCoords
                                     {x= a
                                     ,y= b
                                     ,w= (abstand1D mPos.x (stringToInt a))
                                     ,h= (abstand1D mPos.y (stringToInt b))
                                     } 
                       in { model | saved = (Rectangle coords.x coords.y coords.w coords.h)::model.saved
                                  , modal = NoModal
                          }
              Polygon   pList   -> { model | modal = DrawShape 
                                                      (Polygon (( String.fromInt mPos.x
                                                                , String.fromInt mPos.y
                                                                )::pList
                                                               )
                                                      )
                                                      mPos
                                   }
      SaveAndCloseModal -> { model | saved = ((getInputShapeFromModal model.modal)::model.saved)
                                   , modal = NoModal
                           }
      LoadShapes result -> 
        case result of
          Ok ""           -> { model | shapesLoadResult = Err (Http.BadBody "Empty result for LoadShapes") }
          Ok "LoadExample"-> model
          Ok "Fenster"    -> model
          Ok "T%C3%BCren" -> model --%C3%BC = ü
          Ok s  -> { model | shapesLoadResult = result
                           , saved = shapesDecoder s
                   }
          Err e -> { model | shapesLoadResult = result }
      ShapesLoaded result ->
        case result of 
          Ok shapes-> { model | saved = shapes, shapesLoadResult = Ok "shapes"}
          Err e    -> { model | saved = [] , shapesLoadResult = (Err e )}
      LinkClicked urlRequest -> model
      UrlChanged url         -> { model | url = url}
   ,case msg of 
      OpenModal      _   -> Cmd.none
      OnInputModal   _ _ -> Cmd.none
      CloseModal         -> Cmd.none
      SvgShapeHover  _   -> Cmd.none
      UpdateMousePos _   -> Cmd.none
      InputDrawShape     -> Cmd.none
      SaveAndCloseModal  -> Cmd.none 
      ShapesLoaded   _   -> Cmd.none  
      LoadShapes result  -> 
        case result of
          Ok "LoadExample" ->
            Http.get 
              { url = "https://cors-anywhere.herokuapp.com/"
                    --++"https://users.informatik.uni-halle.de/~hinnebur/shapes-circ-rect-ploy.json" --don't works yet
                    ++"https://users.informatik.uni-halle.de/~hinnebur/shapes-list-circ-rect-poly.json"
                    --++"https://users.informatik.uni-halle.de/~hinnebur/shapes-list-circ-rect.json"
              , expect = Http.expectString LoadShapes
              }   
          Ok s  -> if (s=="Fenster" || s=="T%C3%BCren") --%C3%BC = ü
                     then Http.get
                            { url = "https://cors-anywhere.herokuapp.com/" ++        (if (s=="Fenster") then
                                    "https://users.informatik.uni-halle.de/~hinnebur/fenster-liste.json"else
                                    "https://users.informatik.uni-halle.de/~hinnebur/tueren-liste.json")
                            , expect = Http.expectJson ShapesLoaded rectListDecoder
                            }
                     else Cmd.none
          Err e -> Cmd.none
      LinkClicked urlReq -> 
        case urlReq of
          Browser.Internal url  -> Nav.pushUrl model.key (Url.toString url)

          Browser.External href -> Nav.load href      
      UrlChanged     url -> Tuple.second ( update (LoadShapes (Ok (Maybe.withDefault  "" url.fragment)))
                                                  model
                                         )   
   )
-- Parts of the UPDATE
updateModal : ModalMsg -> Model -> Model
updateModal msg model =
  case msg of
    Check shape -> if (eq_String_Modal "ShowInput" model.modal)
                     then if (eq_String_Shape shape (getInputShapeFromModal model.modal))
                            then { model | modal = ShowInput Empty "" "" }
                            else { model | modal = ShowInput (shapeDefaultConstruktor shape) "" "" }
                     else model    
    Input inputType value -> 
      case model.modal of
        NoModal         -> model -- ERROR
        ShowTable       -> model -- ERROR       
        ShowShape _     -> model -- ERROR
        DrawShape iShape mouse -> updateInput inputType value model "DrawShape" iShape "" "" mouse
        ShowInput iShape ix iy -> updateInput inputType value model "ShowInput" iShape ix iy {x = 0, y = 0}
    InputPoint -> case model.modal of
                    NoModal           -> model -- ERROR
                    ShowTable         -> model -- ERROR       
                    ShowShape _       -> model -- ERROR                    
                    DrawShape _ _ -> model -- ERROR?
                    ShowInput iShape ix iy -> 
                      { model | modal = ShowInput (Polygon ((ix,iy)::(getPointListFromPolygon iShape))) "" "" }
    AddShape   -> { model | saved = (getInputShapeFromModal model.modal)::model.saved
                          , modal = ShowInput 
                                      (shapeDefaultConstruktor
                                        (shapeNameToString
                                          (getInputShapeFromModal model.modal)
                                        )
                                      ) "" ""
                  }
    DeleteShape id -> { model | saved = List.append (List.take (id-1) model.saved) 
                                                    (List.drop (id) model.saved)
                      }
    OpenShowInput    -> if (eq_String_Modal "ShowInput" model.modal)
                          then { model | modal = NoModal }
                          else { model | modal = ShowInput Empty "" "" } 
    OpenShowTable    -> if (eq_String_Modal "ShowTable" model.modal)
                          then { model | modal = NoModal   }
                          else { model | modal = ShowTable }
    OpenShowShape id -> if (id == 0)
                          then { model | modal = NoModal     }
                          else { model | modal = ShowShape id}
    OpenDrawShape shape  -> { model | modal = DrawShape shape {x = 0, y = 0} }
updateInput : Char -> String -> Model -> String -> Shape -> String -> String -> MousePosition -> Model
updateInput inputType value model modalType iShape ix iy mouse =
    case inputType of
      '1' -> case iShape of
        Empty              ->   model -- ERROR
        Circle     _ b c   -> { model | modal = modalInputConstruktor modalType (Circle    value b c  ) ix iy mouse}
        Rectangle  _ b c d -> { model | modal = modalInputConstruktor modalType (Rectangle value b c d) ix iy mouse}
        Polygon    _       -> { model | modal = modalInputConstruktor modalType iShape               value iy mouse}
      '2' -> case iShape of
        Empty              ->   model -- ERROR
        Circle     a _ c   -> { model | modal = modalInputConstruktor modalType (Circle    a value c  ) ix iy mouse}
        Rectangle  a _ c d -> { model | modal = modalInputConstruktor modalType (Rectangle a value c d) ix iy mouse}
        Polygon    _       -> { model | modal = modalInputConstruktor modalType iShape               ix value mouse}
      '3' -> case iShape of
        Empty              ->   model -- ERROR
        Circle     a b _   -> { model | modal = modalInputConstruktor modalType (Circle    a b value  ) ix iy mouse}
        Rectangle  a b _ d -> { model | modal = modalInputConstruktor modalType (Rectangle a b value d) ix iy mouse}
        Polygon    _       ->   model -- ERROR
      '4' -> case iShape of
        Empty              ->   model -- ERROR
        Circle     _ _ _   ->   model -- ERROR
        Rectangle  a b c _ -> { model | modal = modalInputConstruktor modalType (Rectangle a b c value) ix iy mouse}
        Polygon    _       ->   model -- ERROR
      _   -> model -- ERROR