module Util.ShapeManager exposing (..)

import Json.Decode as JD

type Shape 
  = Empty
  | Circle String String String
  | Rectangle String String String String
  | Polygon (List ( String, String ) ) 

-- Shape Construktors
shapeDefaultConstruktor : String -> Shape
shapeDefaultConstruktor shape =
    case shape of 
      "Circle"   -> Circle    "" "" ""
      "Rectangle"-> Rectangle "" "" "" ""
      "Polygon"  -> Polygon   []
      _          -> Empty
shapesDecoder : String -> List Shape
shapesDecoder s = 
    if (String.contains "shapes" s)
      then []
      else
        List.concat
          [ if (String.contains "circle" s) 
              then [ Circle (shapesDecoderList "circle" "x" s) 
                            (shapesDecoderList "circle" "y" s)
                            (shapesDecoderList "circle" "r" s)
                   ]         
              else []
          , if (String.contains "rectangle" s)  
              then [ Rectangle (shapesDecoderList "rectangle" "x"      s)
                               (shapesDecoderList "rectangle" "y"      s)
                               (shapesDecoderList "rectangle" "width"  s)
                               (shapesDecoderList "rectangle" "height" s)
                   ]            
              else []
          , if (String.contains "polygon" s)
              then [ Polygon (List.map2 Tuple.pair
                             (shapesDecoderListPoly "x" s)
                             (shapesDecoderListPoly "y" s)
                             )
                   ]
              else []
          ]
-- Operations on Shapes
eq_String_Shape : String -> Shape -> Bool
eq_String_Shape string shape = string==(shapeNameToString shape)
-- Get Methods : Parameter of Shapes
getPointListFromPolygon : Shape -> List (String,String) 
getPointListFromPolygon shape =
    case shape of 
      Empty               -> [] 
      Circle    _ _ _     -> [] 
      Rectangle _ _ _ _   -> [] 
      Polygon   pointList -> pointList
-- Transform Shape to other Types
shapeNameToString : Shape -> String
shapeNameToString shape =
    case shape of 
      Empty             -> ""
      Circle    _ _ _   -> "Circle"
      Rectangle _ _ _ _ -> "Rectangle"
      Polygon   _       -> "Polygon"
shapeNameStringToGerman : String -> String
shapeNameStringToGerman shape =
    case shape of 
      "Circle"   -> "Kreis"
      "Rectangle"-> "Rechteck"
      "Polygon"  -> "Vieleck"
      _          -> ""
shapeCoordsToString : Shape -> String
shapeCoordsToString shape =
    case shape of
      Empty                                       -> ""
      Circle    x_center y_center radius          -> x_center++","++y_center++","++radius
      Rectangle x_left   y_upper  x_right y_lower -> x_left++","++y_upper++","++x_right++","++y_lower
      Polygon   pointList                         -> 
          case pointList of
            []        -> ""
            (x,y)::xs -> "("++x++","++y++") "++(shapeCoordsToString (Polygon xs))
shapeCoordsToHtmlCoords : Shape -> String   
shapeCoordsToHtmlCoords shape = 
    case shape of 
      Empty             -> ""
      Circle    _ _ _   -> shapeCoordsToString shape
      Rectangle _ _ _ _ -> shapeCoordsToString shape
      Polygon   _       -> String.left ((String.length (shapeCoordsToHtmlCoordsHelper shape))-1)
                                       (shapeCoordsToHtmlCoordsHelper shape)
shapeCoordsToHtmlCoordsHelper : Shape -> String 
shapeCoordsToHtmlCoordsHelper shape =
    case shape of 
      Empty               -> ""
      Circle    _ _ _     -> shapeCoordsToString shape
      Rectangle _ _ _ _   -> shapeCoordsToString shape
      Polygon   pointList -> 
          case pointList of
            []        -> ""
            (x,y)::xs -> x++","++y++","++(shapeCoordsToHtmlCoordsHelper (Polygon xs))
-- Decoder of shapes
rectListDecoder : JD.Decoder (List Shape)
rectListDecoder = JD.field "rectangle" (JD.list (JD.map4
                                                  Rectangle
                                                    (JD.field "x"      JD.string)
                                                    (JD.field "y"      JD.string)
                                                    (JD.field "width"  JD.string)
                                                    (JD.field "height" JD.string)
                                                )
                                       )  
shapesDecoderList : String -> String -> String -> String
shapesDecoderList shape var s = 
    let res = JD.decodeString
                (JD.field shape (JD.list (JD.field var JD.string)))
                s
    in case res of
         Ok stringList -> case stringList of
                            []       -> ""
                            x::[]    -> x
                            x::xs    -> x++" This List is too long"
         Err e         -> "ERROR"
shapesDecoderListPoly : String -> String -> List String
shapesDecoderListPoly var s =
    let res = JD.decodeString
                (JD.field "polygon" (JD.list (JD.field "points" (JD.list (JD.field var JD.string)))))
                s
    in case res of 
         Ok list -> case list of
                      []    -> []
                      x::[] -> x
                      x::xs -> x++["This List is too long"]
         Err e   -> ["ERROR"]       