module Util.Util exposing (..)
-- Json
import Json.Decode as JD
-- Svg
import Svg
import Svg.Events
-- Http
import Http
-- Html
import Html exposing (Html, Attribute, node)

-- Html Elements
link : List (Attribute msg) -> List (Html msg) -> Html msg
link attributes children = node "link" attributes children
-- Working with Error
   -- Transform Error to other Types
httpErrorToString : Http.Error -> String
httpErrorToString e = "Http.Error: "++ 
    case e of
      Http.BadUrl       string -> "BadUrl "   ++string
      Http.Timeout             -> "Timeout"
      Http.NetworkError        -> "NetworkError"
      Http.BadStatus    int    -> "badStatus "++(String.fromInt int)
      Http.BadBody      string -> "BadBody "  ++string
-- Math Methods
abstand1D : Int -> Int -> String
abstand1D a b = String.fromInt (a-b)
abstand : String -> String -> MousePosition -> String
abstand x y mPos = String.fromInt (round((toFloat (((stringToInt x)-mPos.x)^2+((stringToInt y)-mPos.y)^2) )^0.5))
type alias RectCoords =
  { x : String
  , y : String
  , w : String
  , h : String
  }
manageRectCoords : RectCoords -> RectCoords
manageRectCoords coords =
    let x = stringToInt coords.x
        y = stringToInt coords.y
        w = stringToInt coords.w
        h = stringToInt coords.h
    in if w>0
         then if h>0
                then coords
                else {x=String.fromInt  x   ,y=String.fromInt (y+h),w=String.fromInt  w,h=String.fromInt -h}
         else if h>0
                then {x=String.fromInt (x+w),y=String.fromInt  y   ,w=String.fromInt -w,h=String.fromInt  h}
                else {x=String.fromInt (x+w),y=String.fromInt (y+h),w=String.fromInt -w,h=String.fromInt -h}
-- Transformation between elm-types
stringToInt : String -> Int
stringToInt a = Maybe.withDefault 0 (String.toInt a)
-- MouseMove
type alias MousePosition =
    { x : Int, y : Int }   
onMouseMove : (MousePosition -> msg) -> Svg.Attribute msg
onMouseMove mapMousePositionToMsg =
    Svg.Events.on "mousemove" (JD.map mapMousePositionToMsg offsetMousePosition)
offsetMousePosition : JD.Decoder MousePosition
offsetMousePosition =
    JD.map2 MousePosition (JD.field "offsetX" JD.int)
                          (JD.field "offsetY" JD.int)